package de.fhdw.gaming.ipspiel23.ht.strategy.impl.mixed;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.Random;
import java.util.function.Supplier;

import de.fhdw.gaming.core.domain.GameException;
import de.fhdw.gaming.ipspiel23.ht.domain.IHTPlayer;
import de.fhdw.gaming.ipspiel23.ht.domain.IHTState;
import de.fhdw.gaming.ipspiel23.ht.moves.IHTMove;
import de.fhdw.gaming.ipspiel23.ht.moves.factory.IHTMoveFactory;
import de.fhdw.gaming.ipspiel23.ht.strategy.impl.HTStrategy;

/**
 * 
 * implements {@link HTStrategy} by choosing one of the three possible answers.
 *
 */
public class HTMixedStrategy extends HTStrategy {

    /**
     * Random to determine the Answer to Choose.
     */
    private static final Random RANDOM = new Random();

    /**
     * Constructor.
     * 
     * @param moveFactory moveFactory.
     */
    protected HTMixedStrategy(final IHTMoveFactory moveFactory) {
        super(moveFactory);
    }

    @Override
    public Optional<IHTMove> computeNextMove(final int gameId, final IHTPlayer player, final IHTState state)
            throws GameException, InterruptedException {
        final IHTMoveFactory moveFactory = this.getMoveFactory();
        final List<Supplier<IHTMove>> moveSuppliers = new ArrayList<>(3);
        moveSuppliers.add(moveFactory::createHeadsMove);
        moveSuppliers.add(moveFactory::createTailsMove);
        moveSuppliers.add(moveFactory::createEdgeMove);
        return Optional.of(moveSuppliers.get(RANDOM.nextInt(moveSuppliers.size())).get());
    }
}
